from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from pymongo import MongoClient
from firebase_admin import credentials, firestore, initialize_app
import logging
from settings import DEBUG, DATABASE_TYPE, FIREBASE_CREDENTIALS_PATH, DB_NAME, DB_USER, DB_PASSWORD, DB_HOST, DB_PORT

# Set up logging configuration
logging.basicConfig(level=logging.DEBUG if DEBUG else logging.INFO)
logger = logging.getLogger(__name__)

# Initialize the Base for SQLAlchemy ORM
Base = declarative_base()

# Database URL configuration based on DATABASE_TYPE
if DATABASE_TYPE == 'sqlite':
    DATABASE_URL = f'sqlite:///db.sqlite3'
elif DATABASE_TYPE == 'mysql':
    DATABASE_URL = f'mysql+pymysql://{DB_USER}:{DB_PASSWORD}@{DB_HOST}/{DB_NAME}'
elif DATABASE_TYPE == 'postgresql':
    DATABASE_URL = f'postgresql+psycopg2://{DB_USER}:{DB_PASSWORD}@{DB_HOST}:{DB_PORT}/{DB_NAME}'
elif DATABASE_TYPE == 'mongodb':
    DATABASE_URL = f'mongodb://{DB_HOST}:{DB_PORT}/'
elif DATABASE_TYPE == 'firebase':
    DATABASE_URL = 'firebase'
else:
    raise ValueError("Unsupported database type specified in settings.")

# SQLAlchemy engine and session for relational databases
if DATABASE_TYPE in ['sqlite', 'mysql', 'postgresql']:
    try:
        # Initialize SQLAlchemy engine
        engine = create_engine(DATABASE_URL, echo=DEBUG)
        SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)
        logger.info(f"{DATABASE_TYPE.capitalize()} database engine created successfully.")
    except Exception as e:
        logger.error("Error creating SQLAlchemy engine: %s", e)
        engine = None
        SessionLocal = None
else:
    # Set engine and SessionLocal to None for non-relational databases
    engine = None
    SessionLocal = None

# Helper function for database session management
def get_db():
    """
    Yields a database session object for SQLAlchemy-supported databases.
    For MongoDB, returns a pymongo database client.
    For Firebase, returns a firestore client.
    
    Usage:
    ```
    with get_db() as db:
        # perform database operations
    ```
    """
    # SQLAlchemy-supported relational databases
    if DATABASE_TYPE in ['sqlite', 'mysql', 'postgresql']:
        db = SessionLocal()
        try:
            logger.info("SQLAlchemy session started.")
            yield db
        except Exception as e:
            logger.error("An error occurred with the SQLAlchemy session: %s", e)
            raise  # Reraise the exception to handle it at a higher level if needed
        finally:
            db.close()
            logger.info("SQLAlchemy session closed.")

    # MongoDB non-relational database
    elif DATABASE_TYPE == 'mongodb':
        try:
            client = MongoClient(DATABASE_URL)
            db = client[DB_NAME]  # Use the specified database
            logger.info("MongoDB client connected successfully.")
            yield db
        except Exception as e:
            logger.error("Error connecting to MongoDB: %s", e)
            raise
        finally:
            client.close()
            logger.info("MongoDB client connection closed.")

    # Firebase non-relational database
    elif DATABASE_TYPE == 'firebase':
        try:
            if not firebase_admin._apps:
                # Initialize Firebase app only once
                cred = credentials.Certificate(FIREBASE_CREDENTIALS_PATH)
                initialize_app(cred)
                logger.info("Firebase app initialized successfully.")
            db = firestore.client()
            yield db
        except Exception as e:
            logger.error("Error connecting to Firebase Firestore: %s", e)
            raise
        finally:
            # Firebase Firestore client does not require manual closing
            logger.info("Firestore client ready (no explicit close required).")

# Initialize SQLAlchemy tables if using a relational database
if DATABASE_TYPE in ['sqlite', 'mysql', 'postgresql']:
    try:
        Base.metadata.create_all(bind=engine)
        logger.info("All tables created successfully in the database.")
    except Exception as e:
        logger.error("An error occurred while creating tables: %s", e)
